# This tests fnstsw on x86_64

.include "../common/syscalls.x86_64"

	.globl _start	
_start:	

	########################################
	# Test fnstsw
	########################################


	#
	# Check against 0 reg
	#
	
	mov    $fnstsw_test1,%rcx
	call   write_stdout

	xor    %rax,%rax
	
	fnstsw %ax			  # check status word 
	push   %rax
	call   print_hex
	call   print_lf
	mov    $fnstsw_sb1,%rcx
	call   write_stdout	
	pop    %rax

	cmp    $0,%eax
	jne    fail
	
	#
	# Check against full reg
	#

	mov    $fnstsw_test2,%rcx
	call   write_stdout	

	mov    $0xdeadbeefdeadbeef,%rax
	
	fnstsw %ax			  # check status word 
	push   %rax
	call   print_hex
	call   print_lf
	mov    $fnstsw_sb2,%rcx
	call   write_stdout		
	pop    %rax	
	
	cmp    $0xdead0000,%eax
	jne    fail

	#
	# Check against mem
	#
	
	mov    $fnstsw_test3,%rcx
	call   write_stdout	

	xor    %rax,%rax
	fnstsw temp2			  # check status word 
	
	mov    temp2,%eax
	
	push   %rax
	call   print_hex
	call   print_lf
	mov    $fnstsw_sb3,%rcx
	call   write_stdout		
	pop    %rax	
	
	cmp    $0xc0de0000,%eax
	jne    fail

	xor    %rax,%rax

	#
	# Push something on fp stack 8 times
	#

	mov    $8,%rcx
	mov    $(7<<11),%rdx
push_loop:	
	
	fldl   temp		  # push something on fp stack
	fnstsw %ax			  # check status word 

	push   %rcx
	push   %rdx
	push   %rax
	
	mov    $fnstsw_test4,%rcx
	call   write_stdout
	pop    %rax
	push   %rax
	
	call   print_hex
	call   print_lf
	
	mov    $fnstsw_sb4,%rcx
	call   write_stdout
	
	pop    %rbx
	pop    %rax
	push   %rdx
	push   %rbx
	
	mov    %rdx,%rax
	call   print_hex
	call   print_lf
	
	pop    %rax	
	pop    %rdx
	pop    %rcx

	cmp    %rax,%rdx
	jne    fail

	sub    $(1<<11),%rdx

	loop   push_loop
	
	
	#
	# Push something and cause an overflow
	#

	mov    $fnstsw_test5,%rcx
	call   write_stdout

	fldl   temp		  # push something on fp stack
	fnstsw %ax			  # check status word 

	push   %rax
	
	call   print_hex
	call   print_lf
	
	mov    $fnstsw_sb5,%rcx
	call   write_stdout
	
	pop    %rax


	cmp    $0x3a41,%rax
	jne    fail

	mov	$good_string,%rcx
	jmp	write_message

fail:	
	mov	$bad_string,%rcx
write_message:	
	call	write_stdout	


our_exit:

	# fall through to exit

.include "../common/library.x86_64"

.data

temp:	.int 0xdeadbeef,0xcafebabe
temp2:	.int 0xc0dea5a5

fnstsw_test1:    .ascii "Testing fnstsw with empty %rax: \0"
fnstsw_sb1:      .ascii "                     should be: 0000000000000000\n\0"
fnstsw_test2:    .ascii "Testing fnstsw with full %rax : \0"
fnstsw_sb2:      .ascii "                     should be: DEADBEEFDEAD0000\n\0"
fnstsw_test3:    .ascii "Testing fnstsw to mem         : \0"
fnstsw_sb3:      .ascii "                     should be: 00000000C0DE0000\n\0"
fnstsw_test4:    .ascii "Pushing value on stack, status: \0"
fnstsw_sb4:      .ascii "                     should be: \0"
fnstsw_test5:    .ascii "Push on stack causing overflow: \0"
fnstsw_sb5:      .ascii "                     should be: 0000000000003A41\n\0"




good_string:	.ascii	"Good\n\0"
bad_string:	.ascii	"Fail\n\0"
